package ch.inf3.indexation.importer;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import javax.json.Json;
import javax.json.JsonArray;
import javax.json.JsonException;
import javax.json.JsonObject;
import javax.json.JsonValue;
import javax.json.stream.JsonParser;

public class BookmarksJsonImporter {

    public static final String TYPE_PLACE = "text/x-moz-place";

    private final Collection<BookmarkJsonPage> pages;

    public BookmarksJsonImporter(InputStream is) throws JsonException {
        pages = new ArrayList<>(512);

        JsonParser parser = Json.createParser(is);

        // The first element in the export is always the root element that contains
        // a number of folders.
        parser.next();
        JsonArray rootChildren = parser.getObject().getJsonArray("children");

        parseChildren(rootChildren);
    }

    private void parseChildren(JsonArray childrenArray) {
        parseChildren("", childrenArray);
    }

    private void parseChildren(String prefix, JsonArray childrenArray) {
        for (JsonValue childObject : childrenArray) {
            JsonObject entry = childObject.asJsonObject();

            if (TYPE_PLACE.equals(entry.getString("type"))) {
                BookmarkJsonPage page = new BookmarkJsonPage(prefix, entry);
                pages.add(page);
            }

            if (entry.containsKey("children")) {
                String subprefix = prefix + "/" + entry.getString("title");
                parseChildren(subprefix, entry.getJsonArray("children"));
            }
        }
    }

    public Collection<BookmarkJsonPage> getEntries() {
        return Collections.unmodifiableCollection(pages);
    }

}
