package ch.inf3.indexation.server.health;

import ch.inf3.indexation.model.Bookmark;
import java.util.List;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceException;
import org.eclipse.microprofile.health.HealthCheck;
import org.eclipse.microprofile.health.HealthCheckResponse;
import org.eclipse.microprofile.health.HealthCheckResponseBuilder;
import org.eclipse.microprofile.health.Liveness;

@Liveness
@ApplicationScoped
public class ServiceHealthCheck implements HealthCheck {

    @Inject
    private EntityManager em;

    @Override
    public HealthCheckResponse call() {
        HealthCheckResponseBuilder healthBuilder = HealthCheckResponse.named(ServiceHealthCheck.class.getSimpleName());

        boolean isDatabaseOk = em.isOpen();
        if (isDatabaseOk) {
            try {
                List<Bookmark> result = em.createNamedQuery("Bookmark.findAll", Bookmark.class).getResultList();
            } catch (PersistenceException ex) {
                isDatabaseOk = false;
            }
        }

        if (em.isOpen() && isDatabaseOk) {
            healthBuilder = healthBuilder.up();
        } else {
            healthBuilder = healthBuilder.down();
            healthBuilder.withData("cause", "EntityManager is unavailable");
        }

        healthBuilder.name("INF3 Indexation server");
        healthBuilder.withData("since", "2019-10-31 08:34:22");

        return healthBuilder.build();
    }
}
