package ch.inf3.indexation.updater;

import ch.inf3.indexation.crawler.PageCrawler;
import ch.inf3.indexation.model.Bookmark;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.enterprise.concurrent.ManagedExecutorService;
import javax.persistence.EntityManager;
import org.apache.solr.client.solrj.impl.HttpSolrClient;

public class UpdateTaskRunnable implements Runnable {

    private static final Logger LOG = Logger.getLogger(UpdateTaskRunnable.class.getName());

    private final EntityManager em;

    private final ConcurrentHashMap<String, Future<Boolean>> currentJobs = new ConcurrentHashMap<>(1024);

    private final ExecutorService executorService;
    private final HttpSolrClient solr;

    public UpdateTaskRunnable(ManagedExecutorService executorService, EntityManager em, HttpSolrClient solr) {
        this.em = em;
        this.executorService = executorService;
        this.solr = solr;
    }

    @Override
    public void run() {
        List<Bookmark> toUpdateList = em.createQuery("SELECT t FROM Bookmark t WHERE t.lastRetrieved IS NULL").getResultList();

        if (toUpdateList.size() > 200) {
            LOG.log(Level.INFO, "The list of entries to update has more than 200 entries, "
                    + "only the first 200 will be updated during this update round.");
            toUpdateList = toUpdateList.subList(0, 200);
        }

        for (Bookmark toUpdate : toUpdateList) {
            currentJobs.computeIfAbsent(toUpdate.getUuid(), e -> {
                // An entity manager must not be shared between multiple threads, we thus build
                // a new one for each thread.
                EntityManager emThread = em.getEntityManagerFactory().createEntityManager();

                PageCrawler crawler = new PageCrawler(emThread, toUpdate.getUuid(), solr, currentJobs);
                return executorService.submit(crawler);
            });
        }
    }
}
